# frozen_string_literal: true

require 'application_system_test_case'

class ArticlesTest < ApplicationSystemTestCase
  setup do
    visit root_path
  end

  test 'a guest User cannot see the new Article link' do
    assert_no_link 'New Article'
  end

  test 'a guest User cannot see the new Article page' do
    visit new_article_path
    assert_title 'Sign In'
  end

  test 'signed in User can see the new Article link' do
    sign_in
    assert_link 'New Article'
  end

  test 'Article can be created' do
    sign_in

    click_link 'New Article'
    assert_title 'New Article'

    fill_in 'Title', with: 'My title'
    fill_in 'Body', with: 'My long body'
    click_on 'Create'

    assert_text 'Article was successfully created.'
    assert_title 'My title'
    assert_text 'Author: existing@example.com'
    assert_text 'My long body'
  end

  test 'validation errors are shown' do
    sign_in

    click_on 'New Article'
    click_on 'Create'

    assert_text "Title can't be blank"
    assert_text "Body can't be blank"
    assert_text 'Body is too short (minimum is 10 characters)'
  end

  test 'previously entered data is shown' do
    sign_in

    click_link 'New Article'
    fill_in 'Body', with: 'Short'
    click_on 'Create'

    assert_text 'Body is too short (minimum is 10 characters)'
    assert_field 'Body', with: 'Short'
  end

  test 'new Article is listed in the index page' do
    sign_in

    click_link 'New Article'
    fill_in 'Title', with: 'My title'
    fill_in 'Body', with: 'My long body'
    click_on 'Create'

    visit root_path
    assert_link 'My title'
    assert_text 'Author: existing@example.com'
    assert_text 'My long body'
  end

  test 'Article can be edited' do
    sign_in

    click_link 'Lorem Barnak'
    click_on 'Edit'

    assert_title 'Edit Article'

    assert_field 'Title', with: 'Lorem Barnak'
    assert_field 'Body', with: 'Cimonaque de sacristi de colon de câlique de '\
      "batèche de p'tit Jésus de charogne de marde de patente à gosse de " \
      "sainte-viarge de bout d'crisse de cibouleau de bâtard."

    fill_in 'Title', with: 'Edited title'
    fill_in 'Body', with: 'Edited body'
    click_on 'Update'

    assert_text 'Article was successfully updated.'
    assert_title 'Edited title'
    assert_text 'Author: existing@example.com'
    assert_text 'Edited body'
  end

  test 'validation errors are shown when editing Article' do
    sign_in

    click_on 'Lorem Barnak'
    click_on 'Edit'

    fill_in 'Title', with: ''
    fill_in 'Body', with: 'Short'
    click_on 'Update'

    assert_text "Title can't be blank"
    assert_text 'Body is too short (minimum is 10 characters)'
    assert_field 'Title', with: ''
    assert_field 'Body', with: 'Short'
  end

  test 'guest User cannot see the edit link' do
    click_link 'Lorem Barnak'
    assert_no_link 'Edit'
  end

  test 'guest User cannot access the edit page directly' do
    click_link 'Lorem Barnak'

    visit "#{current_path}/edit"

    assert_title 'Sign In'
  end

  test 'User who is not the author of the Article cannot see its edit link' do
    sign_in

    click_on 'The Hobbit'
    assert_no_link 'Edit'
  end

  test 'User who is not the author of the Article cannot access the edit page directly' do
    sign_in

    click_link 'The Hobbit'

    visit "#{current_path}/edit"

    assert_title 'Sign In'
  end

  test 'Article can be deleted' do
    sign_in

    click_on 'Lorem Barnak'
    click_on 'Destroy'

    assert_title 'Articles'
    assert_text 'Article was successfully destroyed'
    assert_no_link 'Lorem Barnak'
  end

  test 'guest User cannot see the delete button' do
    click_on 'Lorem Barnak'
    assert_no_button 'Destroy'
  end

  test 'User who is not the author of the Article cannot see its delete link' do
    sign_in

    click_on 'The Hobbit'
    assert_no_button 'Destroy'
  end
end
