# frozen_string_literal: true

require 'test_helper'

require 'upgrow/active_record_conversion'
require 'upgrow/basic_model'
require 'upgrow/basic_repository'
require 'upgrow/input'

module Upgrow
  class ActiveRecordConversionTest < ActiveSupport::TestCase
    class UserRecord < TestRecord
    end

    class ArticleRecord < TestRecord
      belongs_to :user
    end

    class User < BasicModel
      attribute :name
    end

    class Article < BasicModel
      attribute :title
      belongs_to :user
    end

    class ArticleRepository < BasicRepository
      include ActiveRecordConversion
    end

    setup do
      @repository = ArticleRepository.new
      @record = ArticleRecord.new(id: 1, title: 'The Hobbit')
    end

    test '#to_model converts a Record into a Model' do
      model = @repository.to_model(@record)

      assert_instance_of Article, model
      assert_equal 1, model.id
      assert_equal 'The Hobbit', model.title
    end

    test '#to_model converts an array of Records into an array of Models' do
      second_record = ArticleRecord.new(id: 2, title: 'Harry Potter')

      models = @repository.to_model([@record, second_record])

      assert_equal 2, models.size

      assert_instance_of Article, models.first
      assert_equal 'The Hobbit', models.first.title
      assert_equal 1, models.first.id

      assert_instance_of Article, models.last
      assert_equal 'Harry Potter', models.last.title
      assert_equal 2, models.last.id
    end

    test '#to_model adds any loaded assocation to the resulting Model' do
      user_record = UserRecord.new(id: 666, name: 'JRR Tolkien')

      record = ArticleRecord.new(
        id: 1, title: 'The Hobbit', user: user_record
      )

      model = @repository.to_model(record)

      assert_instance_of User, model.user
      assert_equal 666, model.user.id
      assert_equal 'JRR Tolkien', model.user.name
    end

    test '#to_model does not add unloaded assocations to the resulting Model' do
      model = @repository.to_model(@record)

      assert_raises(BasicModel::AssociationNotLoadedError) { model.user }
    end

    test '#to_model is nil when the given object is nil' do
      assert_nil @repository.to_model(nil)
    end
  end
end
