# frozen_string_literal: true

require 'test_helper'
require 'rails/generators/test_case'
require 'generators/upgrow/install/install_generator'

module Upgrow
  module Generators
    class InstallGeneratorTest < Rails::Generators::TestCase
      tests InstallGenerator
      destination File.expand_path('tmp')
      setup :prepare_destination

      test 'generate directories and base classes' do
        run_generator

        assert_file 'app/actions/application_action.rb', <<~RUBY
          # frozen_string_literal: true

          class ApplicationAction < Upgrow::Action
          end
        RUBY

        assert_file 'app/inputs/application_input.rb', <<~RUBY
          # frozen_string_literal: true

          class ApplicationInput < Upgrow::Input
          end
        RUBY

        assert_file 'app/models/application_model.rb', <<~RUBY
          # frozen_string_literal: true

          class ApplicationModel < Upgrow::Model
          end
        RUBY

        assert_file 'app/repositories/application_repository.rb', <<~RUBY
          # frozen_string_literal: true

          class ApplicationRepository < Upgrow::Repository
          end
        RUBY
      end

      test 'transform existing Models into Records' do
        create_file 'app/models/application_record.rb', <<~RUBY
          # frozen_string_literal: true

          class ApplicationRecord < ActiveRecord::Base
          end
        RUBY

        create_file 'app/models/article.rb', <<~RUBY
          # frozen_string_literal: true

          class Article < ApplicationRecord
          end
        RUBY

        create_file 'app/models/users/user.rb', <<~RUBY
          # frozen_string_literal: true

          module Users
            class User < ApplicationRecord
            end
          end
        RUBY

        Thor::LineEditor.stub(:readline, 'yes') { run_generator }

        assert_file 'app/records/application_record.rb', <<~RUBY
          # frozen_string_literal: true

          class ApplicationRecord < ActiveRecord::Base
            include Upgrow::Record
          end
        RUBY

        assert_file 'app/records/article_record.rb', <<~RUBY
          # frozen_string_literal: true

          class ArticleRecord < ApplicationRecord
          end
        RUBY

        assert_file 'app/records/users/user_record.rb', <<~RUBY
          # frozen_string_literal: true

          module Users
            class UserRecord < ApplicationRecord
            end
          end
        RUBY

        assert_no_file 'app/models/application_record.rb'
        assert_no_file 'app/models/article.rb'
        assert_no_file 'app/models/users/user.rb'

        assert_file 'app/models/application_model.rb', <<~RUBY
          # frozen_string_literal: true

          class ApplicationModel < Upgrow::Model
          end
        RUBY
      end

      test 'does not transform existing Models into Records when user answers no' do
        create_file 'app/models/application_record.rb', <<~RUBY
          # frozen_string_literal: true

          class ApplicationRecord < ActiveRecord::Base
          end
        RUBY

        create_file 'app/models/article.rb', <<~RUBY
          # frozen_string_literal: true

          class Article < ApplicationRecord
          end
        RUBY

        Thor::LineEditor.stub(:readline, 'no') { run_generator }

        assert_file 'app/models/application_record.rb', <<~RUBY
          # frozen_string_literal: true

          class ApplicationRecord < ActiveRecord::Base
          end
        RUBY

        assert_file 'app/models/article.rb', <<~RUBY
          # frozen_string_literal: true

          class Article < ApplicationRecord
          end
        RUBY

        assert_file 'app/models/application_model.rb', <<~RUBY
          # frozen_string_literal: true

          class ApplicationModel < Upgrow::Model
          end
        RUBY

        assert_no_directory 'app/records'
      end

      private

      def create_file(name, content)
        path = File.expand_path(name, destination_root)
        mkdir_p(File.dirname(path))
        File.write(path, content)
      end
    end
  end
end
