# frozen_string_literal: true

require 'test_helper'

module Upgrow
  class ImmutableObjectTest < ActiveSupport::TestCase
    class Sample < ImmutableObject
      attribute :title
      attribute :body
    end

    class SubSample < Sample
      attribute :name
    end

    setup do
      @original_schema = Sample.schema
    end

    teardown do
      Sample.schema = @original_schema
    end

    test '.schema is empty by default' do
      assert_empty ImmutableObject.schema.attribute_names
    end

    test '.schema can be set' do
      Sample.schema = :my_schema

      assert_equal :my_schema, Sample.schema
    end

    test '.schema contains all declared attributes' do
      assert_equal [:title, :body, :name], SubSample.schema.attribute_names
    end

    test '.new creates a frozen instance with the given attributes' do
      sample = Sample.new(title: 'volmer', body: 'hello')

      assert_equal 'volmer', sample.title
      assert_equal 'hello', sample.body
      assert sample.frozen?
    end

    test 'it does not allow attributes to be mutated' do
      sample = Sample.new(title: 'volmer', body: 'hello')

      refute sample.respond_to?(:title=)
      refute sample.respond_to?(:body=)
    end

    test '.new rejects attributes that do not exist' do
      error = assert_raises(ArgumentError) do
        Sample.new(title: 'volmer', body: 'hello', fake: 'error')
      end

      assert_equal 'Unknown attribute [:fake]', error.message
    end

    test '.new allows inherited attributes' do
      sub_sample = SubSample.new(title: 'volmer', body: 'hello', name: 'rafael')
      assert_equal 'volmer', sub_sample.title
      assert_equal 'hello', sub_sample.body
      assert_equal 'rafael', sub_sample.name
    end

    test '.new allows omitting attributes' do
      sub_sample = SubSample.new(title: 'volmer')
      assert_equal 'volmer', sub_sample.title
      assert_nil sub_sample.body
      assert_nil sub_sample.name
    end

    test '#respond_to? is true for all attribute readers' do
      sub_sample = SubSample.new(title: 'volmer')

      assert sub_sample.respond_to?(:title)
      assert sub_sample.respond_to?(:body)
      assert sub_sample.respond_to?(:name)
    end

    test '#attributes returns the collection of frozen attributes' do
      sample = Sample.new(title: 'volmer', body: 'hello')

      assert_equal({ title: 'volmer', body: 'hello' }, sample.attributes)
      assert sample.attributes.frozen?
    end

    test '#attributes returns attributes that were omitted' do
      sample = Sample.new

      assert_equal({ title: nil, body: nil }, sample.attributes)
    end
  end
end
