# frozen_string_literal: true

require 'test_helper'

require 'active_record'
require 'upgrow/record'

module Upgrow
  class RecordTest < ActiveSupport::TestCase
    class ArticleRecord < ActiveRecord::Base
      include Record
    end

    class BaseRecord < ActiveRecord::Base
      include Record
      self.abstract_class = true
    end

    class UserRecord < BaseRecord; end

    setup do
      @original_table_name = ArticleRecord.table_name
    end

    teardown do
      ArticleRecord.table_name = @original_table_name
    end

    test '.table_name is inferred from the class name without the Record suffix' do
      assert_equal 'articles', ArticleRecord.table_name
    end

    test '.table_name can be set explicitly' do
      ArticleRecord.table_name = 'my_table'
      assert_equal 'my_table', ArticleRecord.table_name
    end

    test '.table_name ignores inherited abstract classes' do
      assert_equal 'users', UserRecord.table_name
    end

    test '.has_many sets a default foreign key based on the class name without the Record suffix' do
      ArticleRecord.has_many(:comments)
      reflection = ArticleRecord.reflect_on_association(:comments)
      assert_equal 'article_id', reflection.foreign_key
    end

    test '.has_many accepts a foreign key value' do
      ArticleRecord.has_many(:cats, foreign_key: 'my_custom_id')
      reflection = ArticleRecord.reflect_on_association(:cats)
      assert_equal 'my_custom_id', reflection.foreign_key
    end

    test '.has_many sets the association class name based on its name and the Record suffix' do
      ArticleRecord.has_many(:paragraphs)
      reflection = ArticleRecord.reflect_on_association(:paragraphs)
      assert_equal 'ParagraphRecord', reflection.options.fetch(:class_name)
    end

    test '.has_many accepts a class name value' do
      ArticleRecord.has_many(:birds, class_name: 'MyClass')
      reflection = ArticleRecord.reflect_on_association(:birds)
      assert_equal 'MyClass', reflection.options.fetch(:class_name)
    end

    test '.has_many sets the foreign key based on the :as option' do
      ArticleRecord.has_many(:words, as: 'writable')
      reflection = ArticleRecord.reflect_on_association(:words)
      assert_equal 'writable_id', reflection.foreign_key
    end

    test '.has_one sets a default foreign key based on the class name without the Record suffix' do
      ArticleRecord.has_one(:excerpt)
      reflection = ArticleRecord.reflect_on_association(:excerpt)
      assert_equal 'article_id', reflection.foreign_key
    end

    test '.has_one accepts a foreign key value' do
      ArticleRecord.has_one(:title, foreign_key: 'my_custom_id')
      reflection = ArticleRecord.reflect_on_association(:title)
      assert_equal 'my_custom_id', reflection.foreign_key
    end

    test '.has_one sets the association class name based on its name and the Record suffix' do
      ArticleRecord.has_one(:signature)
      reflection = ArticleRecord.reflect_on_association(:signature)
      assert_equal 'SignatureRecord', reflection.options.fetch(:class_name)
    end

    test '.has_one accepts a class name value' do
      ArticleRecord.has_one(:image, class_name: 'MyClass')
      reflection = ArticleRecord.reflect_on_association(:image)
      assert_equal 'MyClass', reflection.options.fetch(:class_name)
    end

    test '.has_one sets the foreign key based on the :as option' do
      ArticleRecord.has_one(:pin, as: 'pinnable')
      reflection = ArticleRecord.reflect_on_association(:pin)
      assert_equal 'pinnable_id', reflection.foreign_key
    end

    test '.belongs_to sets a default foreign key based on the association name' do
      ArticleRecord.belongs_to(:user)
      reflection = ArticleRecord.reflect_on_association(:user)
      assert_equal 'user_id', reflection.foreign_key
    end

    test '.belongs_to accepts a foreign key value' do
      ArticleRecord.belongs_to(:book, foreign_key: 'custom_id')
      reflection = ArticleRecord.reflect_on_association(:book)
      assert_equal 'custom_id', reflection.foreign_key
    end

    test '.belongs_to sets the association class name based on its name and the Record suffix' do
      ArticleRecord.belongs_to(:category)
      reflection = ArticleRecord.reflect_on_association(:category)
      assert_equal 'CategoryRecord', reflection.options.fetch(:class_name)
    end

    test '.belongs_to accepts a class name value' do
      ArticleRecord.belongs_to(:section, class_name: 'MyClass')
      reflection = ArticleRecord.reflect_on_association(:section)
      assert_equal 'MyClass', reflection.options.fetch(:class_name)
    end

    test '.has_and_belongs_to_many sets a default foreign key based on the class name without the Record suffix' do
      ArticleRecord.has_and_belongs_to_many(:tags)
      reflection = ArticleRecord.reflect_on_association(:tags)
      assert_equal 'article_id', reflection.foreign_key
    end

    test '.has_and_belongs_to_many accepts a foreign key value' do
      ArticleRecord.has_and_belongs_to_many(:stars, foreign_key: 'my_custom_id')
      reflection = ArticleRecord.reflect_on_association(:stars)
      assert_equal 'my_custom_id', reflection.foreign_key
    end

    test '.has_and_belongs_to_many sets a default association foreign key based on the class name without the Record suffix' do
      ArticleRecord.has_and_belongs_to_many(:prizes)
      reflection = ArticleRecord.reflect_on_association(:prizes)
      assert_equal 'prize_id', reflection.association_foreign_key
    end

    test '.has_and_belongs_to_many sets a default association foreign key based on the given class name' do
      ArticleRecord.has_and_belongs_to_many(:trees, class_name: 'MyClass')
      reflection = ArticleRecord.reflect_on_association(:trees)
      assert_equal 'my_class_id', reflection.association_foreign_key
    end

    test '.has_and_belongs_to_many sets a default association foreign key based on the given class name without the Record suffix' do
      ArticleRecord.has_and_belongs_to_many(:dogs, class_name: 'MyClassRecord')
      reflection = ArticleRecord.reflect_on_association(:dogs)
      assert_equal 'my_class_id', reflection.association_foreign_key
    end

    test '.has_and_belongs_to_many accepts an association foreign key value' do
      ArticleRecord.has_and_belongs_to_many(
        :countries, association_foreign_key: 'my_custom_id'
      )
      reflection = ArticleRecord.reflect_on_association(:countries)
      assert_equal 'my_custom_id', reflection.association_foreign_key
    end

    test '.has_and_belongs_to_many sets the association class name based on its name and the Record suffix' do
      ArticleRecord.has_and_belongs_to_many(:flags)
      reflection = ArticleRecord.reflect_on_association(:flags)
      assert_equal 'FlagRecord', reflection.options.fetch(:class_name)
    end

    test '.has_and_belongs_to_many accepts a class name value' do
      ArticleRecord.has_and_belongs_to_many(:horses, class_name: 'MyClass')
      reflection = ArticleRecord.reflect_on_association(:horses)
      assert_equal 'MyClass', reflection.options.fetch(:class_name)
    end
  end
end
